//
// Name: FTCOM.PRG
// Project: Fischer Techniek interfacing
// Description: Interface-driver for II(30402) & RoboPro in II(30402)-mode
// Language: Clipper 5.2
// Author: Rene Nooteboom
// Date created: 27-07-2005
// Date updated: 29-08-2005
//
//  default values :
//
//  serial port   I/O    IRQ
//
//   COM1         3F8H    4
//   COM2         2F8H    3
//   COM3         3E8H    4
//   COM4         2E8H    3
//
//  note : -  uses serial comm. functions in catools library
//
//   example ftcom.ini file :  //
//                             //
//                             COM3_IO = hE48
//                             COM3_IO = E48 assumes hex
//                             COM2_IO = 899 assumes decimal
//                             COM2_IO = h899 hex !
//                             COM3_IRQ = 10 assumes decimal
//                             COM1_IRQ = hB
//                             COM1_IRQ = B : mag ook
//                             COM4_IRQ = 0 : means don't set IRQ
//                             COM4_IRQ = - : also don't set IRQ
//                             //
//
#include "inkey.ch"
#include "generic.ch"
#include "noot.ch"
#include "ft.ch"
//
STATIC nMotorByte_:={0,0,0,0} // max 4 comports
STATIC lComUsed_ := {FALSE,FALSE,FALSE,FALSE}
STATIC cHexComIO__ := {"3F8","2F8","3E8","2E8"}
//
FUNCTION FtCInit(nComPort,xIOAddress,xIRQ,laskvalues)
//
// all arguments optional,xIOAddress and xIRQ can be
// numeric decimal or a hex-string
// if xIRQ = 0 do not set IRQ
//
 LOCAL cmsg,getlist:={}
 DEFAULT nComPort TO 1,laskvalues TO TRUE
 IF xIOAddress == NIL
  xIOAddress := FtCGetIOAddress(nComPort)
 ENDIF
 IF xIRQ == NIL
  xIRQ := FtCGetIRQ(nComPort)
 ENDIF
 IF IS_NUM(xIOAddress) // convert to hex
  xIOAddress := padl(alltrim(ntoc(xIOAddress,16,3)),3,"0")
 ENDIF
 IF IS_NUM(xIRQ) // convert to hex
  xIRQ := alltrim(ntoc(xIRQ,16,1))
 ENDIF
 IF laskvalues
  save_env({9,5,13,40})
  color_set(CR_GETS)
  @ 9,5 CLEAR TO 13,40
  @ 9,5 TO 13,40
  @ 10,6 SAY "COM-port(1 to 4) :"
  @ 11,6 SAY "IO-Address(hex)  :"
  @ 12,6 SAY "IRQ(hex)         :"
  @ 10,30 GET nComPort PICTURE "9" VALID IS_BETW(nComPort,1,4)
  @ 11,30 GETX xIOAddress MESSAGE "hexadecimal" PICTURE "@! XXX" ;
   VALID IS_HEX(xIOAddress) ERROR "not hex"
  @ 12,30 GETX xIRQ MESSAGE "hexadecimal(0=is do not set irq)" ;
   PICTURE "@! X" VALID IS_HEX(xIRQ) ERROR "not hex"
  READ
  rest_env()
  IF lastkey() == K_ESC
   nComPort := -1
  ENDIF
 ENDIF
 IF nComPort > 0
  com_setio(nComPort,xIOAddress)
  cHexComIO__[nComPort] := xIOAddress
  IF !(empty(xIRQ) .OR. xIRQ == "0")
   com_setirq(nComPort,xIRQ)
  ENDIF
  IF com_open(nComPort)
   com_hard(nComPort,FALSE) // no hardware handshake
   IF com_init(nComPort,9600,"N",8,1)
    com_dtr(nComPort,TRUE) // activate DTR signal
// com_rts(nComPort,TRUE)
// com_sflush(nComPort)
    com_flush(nComPort)
    lComUsed_[nComPort] := TRUE
    FtCOutput(nComPort,0) // switch all motors off
    cmsg := "FtCInit(): initialized COM" + str(nComPort,1,0) + " ,IO: " + ;
     xIOAddress + "h, IRQ: " + xIRQ + "h"
    rowmessage(cmsg,,,CR_MESSAGE,2)
    FTCShowUsedIF()
   ELSE
    cmsg := "FtCInit(): failed to initialize COM" + str(nComPort,1,0)
    rowmessage(cmsg,,,CR_WARNING,4,TRUE)
    nComPort := -1
   ENDIF
  ELSE
   cmsg := "FtCInit(): failed to open COM" + str(nComPort,1,0)
   rowmessage(cmsg,,,CR_WARNING,4,TRUE)
   nComPort := -1
  ENDIF
 ELSE
  nComPort := -1
 ENDIF
RETURN nComPort
//
PROCEDURE FtCClose(nComPort)
 DEFAULT nComPort TO 1
 IF nComPort > 0 .AND. lComUsed_[nComPort]
  FtCOutput(nComPort,0)
  com_flush(nComPort)
// com_sflush(nComPort)
  com_close(nComPort)
  lComUsed_[nComPort] := FALSE
 ENDIF
RETURN
//
FUNCTION FtCInput(nComPort,nOutputByte) // both optional
 LOCAL lsended:=FALSE,cCommand,cInputByte:=chr(0),cmsg,lOutput:=FALSE
 DEFAULT nComPort TO 1
 IF nOutputByte == NIL
  nOutputByte := nMotorByte_[nComPort]
 ELSE
  lOutput := TRUE
 ENDIF
 com_flush(nComPort) // clear receive buffer
// com_rts(nComPort,TRUE)
// com_dtr(nComPort,TRUE) // activate DTR signal
 cCommand := chr(193) + chr(nOutputByte) // C1h=193d request for inputbyte
 IF lsended := (com_send(nComPort,cCommand) == 0)
  DO WHILE com_count(nComPort) < 1
  ENDDO
  cInputByte := com_read(nComPort,1) // remove chars as they are read
  IF lOutput
   nMotorByte_[nComPort] := nOutputByte
  ENDIF
 ELSE
  cmsg := "FtCInput(): failed to send to COM" + str(nComPort,1)
  rowmessage(cmsg,,,CR_WARNING,4,TRUE)
 ENDIF
RETURN asc(cInputByte)
//
PROCEDURE FtCOutput(nComPort,nByte)
 FtCInput(nComPort,nByte)
RETURN
//
FUNCTION FtCSensor(nComPort,nE1toE8)
 LOCAL nInput:=0,lSensor:=FALSE
 nInput := FtCInput(nComPort)
 lSensor := isbit(nInput,nE1toE8)
RETURN lSensor
//
PROCEDURE FtCMotorsOff(nComPort)
 FtCOutput(nComPort,0)
RETURN
//
PROCEDURE FtCMotor(nComPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nOutputByte
 DEFAULT nComPort TO 1
 nOutputByte := nMotorByte_[nComPort]
 DO CASE
  CASE nMotorNr == 1
   nOutputByte := numand(nOutputByte,"FC") // 1111 1100
   nOutputByte := numor(nOutputByte,nDirection) // no shift
  CASE nMotorNr == 2
   nOutputByte := numand(nOutputByte,"F3") // 1111 0011
   nOutputByte := numor(nOutputByte,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutputByte := numand(nOutputByte,"CF") // 1100 1111
   nOutputByte := numor(nOutputByte,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutputByte := numand(nOutputByte,"3F") // 0011 1111
   nOutputByte := numor(nOutputByte,nDirection*64) // left shift 6 bits
  OTHERWISE
 ENDCASE
 FtCOutput(nComPort,nOutputByte)
RETURN
//
FUNCTION FtCAD(nComPort,nAnalogInput) // EX = 1 , EY =2
 LOCAL cInputBytes,nADvalue:=0,cCommand:="",cmsg,lsended,nOutputByte
 DEFAULT nComPort TO 1,nAnalogInput TO EX
 nOutputByte := nMotorByte_[nComPort]
 com_flush(nComPort) // clear receive buffer
 DO CASE
  CASE nAnalogInput == EX
   cCommand := chr(197)   // 197d=C5h : give inputbyte and EX(2bytes)
  CASE nAnalogInput == EY
   cCommand := chr(201)  //  201d=C9h : give inputbyte and EY(2bytes)
 ENDCASE
// com_dtr(nComPort,TRUE) // activate DTR signal
 cCommand += chr(nOutputByte)
 IF lsended := (com_send(nComPort,cCommand) == 0)
  DO WHILE com_count(nComPort) < 3
  ENDDO
  cInputBytes := com_read(nComPort,3)
  nADvalue := asc(substr(cInputBytes,2,1))*256 + asc(substr(cInputBytes,3,1))
 ELSE
  cmsg := "FtCAD(): failed to send to COM" + str(nComPort,1)
  rowmessage(cmsg,,,CR_WARNING,4,TRUE)
 ENDIF
RETURN nADvalue
//
PROCEDURE FtCButton(nComPort,nE1toE8)
 DO WHILE !FtCSensor(nComPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE FtCButtonPushed(nComPort,nE1toE8)
 DO WHILE FtCSensor(nComPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE FtCOnNr(nComPort,nO1toO8) // alleen voor RoboPro interface
 LOCAL nByte := nMotorByte_[nComPort]
 DEFAULT nComPort TO 1
 nByte := setbit(nByte,nO1toO8)
 FtCOutput(nComPort,nByte)
RETURN
//
PROCEDURE FtCOffNr(nComPort,nO1toO8) // alleen voor RoboPro interface
 LOCAL nByte := nMotorByte_[nComPort]
 DEFAULT nComPort TO 1
 nByte := clearbit(nByte,nO1toO8)
 FtCOutput(nComPort,nByte)
RETURN
//
PROCEDURE FtCMotorLeft(nComPort,nMotorNr)
 FtCMotor(nComPort,nMotorNr,MCCW)
RETURN
//
PROCEDURE FtCMotorRight(nComPort,nMotorNr)
 FtCMotor(nComPort,nMotorNr,MCW)
RETURN
//
PROCEDURE FtCMotorOff(nComPort,nMotorNr)
 FtCMotor(nComPort,nMotorNr,MOFF)
RETURN
//
PROCEDURE FtCMotorOn(nComPort,nMotorNr)
 FtCMotor(nComPort,nMotorNr,MON)
RETURN
//
// sec delay but keeps interfaces alive
//
PROCEDURE FtCDelay(nDelayTime)
 LOCAL nBeginTime := seconds(),i
 DO WHILE TRUE
  IF nBeginTime > seconds()
   nBeginTime -= 86400
  ENDIF
  FOR i:=1 TO 4
   IF lComUsed_[i]
    FtCInput(i)
   ENDIF
  NEXT i
  IF (seconds() - nBeginTime) > nDelayTime
   EXIT
  ENDIF
 ENDDO
RETURN
//
STATIC FUNCTION FtCGetIOAddress(nComPort)
 LOCAL xIOAddress:=NIL,csearch
 DEFAULT nComPort TO 1
 IF file("ftcom.ini")
  readinival("ftcom.ini")
  csearch := "COM" + str(nComPort,1,0) + "_IO" // example: COM3_IO = h2D7
  IF ini_exist(csearch)
   IF !empty(xIOAddress := ini_value(csearch))
    DO CASE
     CASE IS_NUM(xIOAddress)  // assume decimal number
     CASE IS_CHAR(xIOAddress)
      IF upper(left(xIOAddress,1)) == "H" // hex number follows "Hxxx"
       xIOAddress := padl(alltrim(substr(xIOAddress,2)),3,"0")
      ELSE
       IF !IS_HEX(xIOAddress) // als eerste letter geen ABCDEF
        xIOAddress := NIL
       ELSE
        xIOAddress := padl(alltrim(xIOAddress),3,"0")
       ENDIF
      ENDIF
     OTHERWISE
      xIOAddress := NIL
    ENDCASE
   ENDIF
  ENDIF
  clearinival()
 ENDIF
 IF empty(xIOAddress)
  DO CASE
   CASE nComPort == 1
    xIOAddress := "3F8"
   CASE nComPort == 2
    xIOAddress := "2F8"
   CASE nComPort == 3
    xIOAddress := "3E8"
   CASE nComPort == 4
    xIOAddress := "2E8"
   OTHERWISE
    xIOAddress := "3F8"
  ENDCASE
 ENDIF
RETURN xIOAddress
//
STATIC FUNCTION FtCGetIRQ(nComPort)
 LOCAL xIRQ:=NIL,csearch,cFirstChar
 DEFAULT nComPort TO 1
 csearch := "COM" + str(nComPort,1,0) + "_IRQ" // example : COM2_IRQ = hA
 IF file("ftcom.ini")
  readinival("ftcom.ini")
  IF ini_exist(csearch)
   xIRQ := ini_value(csearch)
   DO CASE
    CASE xIRQ == NIL
    CASE IS_NUM(xIRQ)  // assume decimal number
     IF !IS_BETW(xIRQ,0,15)
      xIRQ := NIL
     ENDIF
    CASE IS_CHAR(xIRQ)
     cFirstChar := upper(left(xIRQ,1))
     DO CASE
      CASE cFirstChar == "H" .AND. len(xIRQ) == 2 // hexnumber follows "Hx"
       xIRQ := substr(xIRQ,2)
       IF !IS_HEX(xIRQ)
        xIRQ := NIL
       ENDIF
      CASE cFirstChar $ "ABCDEF" .AND. len(xIRQ) == 1
       xIRQ := cFirstChar
      CASE cFirstChar == "-" .AND. len(xIRQ) == 1
       xIRQ := "0"
      OTHERWISE
       xIRQ := NIL
     ENDCASE
    OTHERWISE
     xIRQ := NIL
   ENDCASE
  ENDIF
  clearinival()
 ENDIF
 IF xIRQ == NIL
  DO CASE
   CASE nComPort == 1
    xIRQ := 4
   CASE nComPort == 2
    xIRQ := 3
   CASE nComPort == 3
    xIRQ := 4
   CASE nComPort == 4
    xIRQ := 3
   OTHERWISE
    xIRQ := 4
  ENDCASE
 ENDIF
RETURN xIRQ
//
STATIC PROCEDURE FTCShowUsedIF()
 LOCAL cmsg:="FT interface uses:",i
 FOR i := 1 TO 4
  IF lComUsed_[i]
   cmsg += " COM" + str(i,1,0) + ",IO:" + cHexComIO_[i] + "h"
  ENDIF
 NEXT
 rowmessage(cmsg,,,CR_MESSAGE,3)
RETURN
//
FUNCTION FtCHexIO(nComPort)
 LOCAL cHexIO:="---"
 DEFAULT nComPort TO 1
 IF nComPort > 0 .AND. lComUsed_[nComPort]
  cHexIO := cHexComIO_[nComPort]
 ENDIF
RETURN cHexIO
//
// einde ftcom.prg
//
